/*TimeManager TimeAssistant
  phi_lipp
  31.01.2020
  This piece of Code is used for a little Device which manage your Worktime
*/

#include <U8g2lib.h>
#include <Wire.h>
#include <DS3232RTC.h>
#include <Bounce2.h>
#include <SPI.h>
#include <ESP8266WiFi.h>
#include <SD.h>
#include <EEPROM.h>


#define modeButton 0
#define startButton 2
#define ledPin 1
#define rtcInterrupt A0
#define chipSelect 15
#define OledReset 3




U8G2_SSD1306_128X32_UNIVISION_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);  // Adafruit ESP8266/32u4/ARM Boards + FeatherWing OLED
DS3232RTC RTC;
Bounce debouncerMode = Bounce();
Bounce debouncerStart = Bounce();

File myFile;

tmElements_t tm;
byte modus, battery;
const byte  fIHour = 11, sIHour = 12, fIMinute = 14, sIMinute = 15, fISecond = 17, sISecond = 18;
unsigned long displayStandbyPreviousMillis, displayOffPreviousMillis, secondCounter, workingHour, workingMinute;
unsigned long batteryDataPreviousMillis, secondCounterAkku, deepSleepPreviousMillis, deepSleepIntervall;
unsigned long displayStandbyIntervall, displayOffIntervall, batteryDataIntervall;
int counterStart, eepromAddr, diagrammCounter;
const int lineSize = 32;
bool timeStart, displayStandby, storeUpdate, pageUpdate, fastPageUpdate, sqwLow, blink, displayOff;
bool timeStartInfo, diagrammActive, checkTimeBeforeStart, resetBattery;
char lineBuffer[lineSize];
char oldLineBuffer[lineSize];
String title = "TimeAssistant", name = "Philipp"; //for the best result, use a name with letters less than 10



struct SDData{
  int Hour;
  int Minute;
  int WeekHour;
  int WeekMinute;
  int MonthHourDiagramm[32];
  int MonthMinuteDiagramm[32];
  int MonthHour;
  int MonthMinute;
};
SDData diffTime;


void initLogo(int, int, int);
//void CheckBattery();
//void SaveBattery();
//void StartFunction();
//void StopFunction();
//void CheckTimeToday();
//void StartPage();
//void TimeTodayPage();
//void TimeYesterdayPage();
//void PrintTimeTodayPage();
//void TimeWeekPage();
//void TimeMonthPage();



void setup() {  

  WiFi.mode(WIFI_OFF);
  WiFi.forceSleepBegin();
  delay(5);

  debouncerMode.attach(modeButton,INPUT_PULLUP);
  debouncerMode.interval(20);  
  debouncerStart.attach(startButton,INPUT_PULLUP);
  debouncerStart.interval(20);
  pinMode(OledReset, OUTPUT);
  pinMode(chipSelect,INPUT);
  pinMode(ledPin,OUTPUT);
  RTC.squareWave(SQWAVE_1_HZ);

  digitalWrite(OledReset, LOW);
  delay(5);
  digitalWrite(OledReset, HIGH);
  delay(5);
  u8g2.begin();
  RTC.begin();
  //setTime(14, 55, 0, 22, 12, 2019);   //set the system time to 23h31m30s on 13Feb2009
  //RTC.set(now()); //set the RTC from the system time
  
  counterStart = 1;
  diagrammCounter = 1;
  eepromAddr = 1;
  displayStandbyIntervall = 65000;
  displayOffIntervall = 120500; 
  batteryDataIntervall = 2500;
  deepSleepIntervall = 3000;

 
  CheckBattery();
  initLogo(117, 16, 10); // arg: int offsetX 115, int offsetY, int animationSpeed 
  u8g2_prepare();
  
}



void loop() {

  debouncerStart.update();
  debouncerMode.update();

  if(debouncerMode.fell()){
    batteryDataPreviousMillis = millis();       
    if(timeStart != true){
      displayOffPreviousMillis = millis();
      if(displayOff){
        u8g2.setPowerSave(0);
        displayOff = false;
      }
      else{
        modus++;
        diagrammCounter = 1;
        diagrammActive = false;        
      }
      storeUpdate = false;
      analogWrite(ledPin, 0);
      if(modus == 5)modus = 0;
      }
    else{
      if(displayStandby){
        u8g2.setPowerSave(0);
        displayStandbyPreviousMillis = millis();
      }
      timeStartInfo = true;
    }
    pageUpdate = false;
    fastPageUpdate = true;    
  }


  if(debouncerStart.fell()){
    displayOffPreviousMillis = millis();
    if(modus == 4){
      diagrammCounter++;
      if(diagrammCounter % 2 == 0)diagrammActive = true;
      else diagrammActive = false;
      pageUpdate = false;
      fastPageUpdate = true;          
    }
    else counterStart++;
    
    if(timeStart != true && (counterStart % 2 == 0) && modus == 0 && displayOff == false){
      displayStandbyPreviousMillis = millis();
      timeStart = true;
      storeUpdate = false;
    }
    if(displayStandby == true){
      u8g2.setPowerSave(0);
      displayStandbyPreviousMillis = millis(); 
    }
    if(displayStandby == false && timeStart == true && (counterStart % 2 != 0)){
      StopFunction();
      timeStart = false;
    }
  }  

  if((millis() - batteryDataPreviousMillis > batteryDataIntervall) && debouncerMode.read() == LOW && !timeStart){
    if(resetBattery){
      battery = 201;
      SaveBattery();  
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_6x10_tf); //6x10
      u8g2.drawStr(10,2,"Battery set to full");
      u8g2.sendBuffer();
      delay(1000);      
    }
    else{
    u8g2.clearBuffer();  
    u8g2.drawStr(10,2,"DeepSleep activated");
    u8g2.setFont(u8g2_font_4x6_tf); //6x10
    u8g2.drawStr(0,18,"stay on Button to reset Battery");
    u8g2.sendBuffer();       
    resetBattery = true;
    batteryDataPreviousMillis = millis();
    deepSleepPreviousMillis = millis();
    } 
  }

  if(((millis() - deepSleepPreviousMillis) > deepSleepIntervall) && resetBattery){
      u8g2.setPowerSave(1);            
      ESP.deepSleep(0);    
  }

  if(millis() - displayStandbyPreviousMillis > displayStandbyIntervall && timeStart == true){
    u8g2.setPowerSave(1);
    displayStandby = true;
    counterStart = 1;
  }
  else displayStandby = false;

  if(((analogRead(rtcInterrupt) < 50 && sqwLow == false) || fastPageUpdate == true) && !resetBattery){
    if(secondCounterAkku++ % 252 == 0){ //108
    battery--;
    SaveBattery();
    }    
    if(timeStart){
      if(!checkTimeBeforeStart){
        CheckTimeToday(); 
      }    
      if(timeStartInfo){
        StartPage();
        delay(2000);
        PrintTimeTodayPage();
        delay(2000);
      }
      else{
        StartFunction();      
      }
    }
    else{  
      if(millis() - displayOffPreviousMillis > displayOffIntervall && displayOff == false){
        u8g2.setPowerSave(1);
        analogWrite(ledPin, 50);
        displayOff = true;
      }      
      switch(modus){
        case 0:
          StartPage();          
          break;
        case 1:
          TimeTodayPage();
          break;
        case 2:
          TimeYesterdayPage();
          break; 
        case 3:
          TimeWeekPage();
          break; 
        case 4:
          TimeMonthPage();
          break;                
      }
      fastPageUpdate = false;
    }
    sqwLow = true;
    fastPageUpdate = false;
  }
  else if(analogRead(rtcInterrupt) > 50){
    sqwLow = false;
  }
 
}



void u8g2_prepare(void) {
  u8g2.setFont(u8g2_font_6x10_tf); //6x10
  u8g2.setFontRefHeightExtendedText();
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setFontDirection(0);
}



void getTime(){
  delay(2);
  RTC.read(tm);
  delay(2);
}



void SaveBattery(){
  EEPROM.begin(1);
  EEPROM.write(eepromAddr,battery);
  EEPROM.end();
}



void CheckBattery(){
  EEPROM.begin(1);
  battery = EEPROM.read(eepromAddr);
  EEPROM.end();
}



void StartPage(){
  getTime();
  u8g2.clearBuffer();
  if(!timeStartInfo){
    u8g2.drawStr(0,0,"Hello");
    u8g2.setCursor(36,0);
    u8g2.print(name);
    if(name.length() > 0 && name.length() < 10){
      u8g2.drawStr(name.length()*6 + 38,0,", it is");    
    }
    else{
      u8g2.drawStr(36,0,"Friend, it is");
    }
  }

  if(tm.Hour < 10){
    u8g2.setCursor(0,12);
    u8g2.print(0);
    u8g2.setCursor(6,12);
    u8g2.print(tm.Hour);
  }
  else{
    u8g2.setCursor(0,12);
    u8g2.print(tm.Hour);
  }
  u8g2.setFont(u8g2_font_5x8_tf);
  u8g2.drawStr(12,12,":");
  u8g2.setFont(u8g2_font_6x10_tf);
  if(tm.Minute < 10){
    u8g2.setCursor(18,12);
    u8g2.print(0);
    u8g2.setCursor(24,12);
    u8g2.print(tm.Minute);
  }
  else{
    u8g2.setCursor(18,12);
    u8g2.print(tm.Minute);
  }
  u8g2.setFont(u8g2_font_5x8_tf);  
  u8g2.drawStr(30,12,":");
  u8g2.setFont(u8g2_font_6x10_tf);  
  if(tm.Second < 10){
    u8g2.setCursor(36,12);
    u8g2.print(0);
    u8g2.setCursor(42,12);
    u8g2.print(tm.Second);
  }
  else{
    u8g2.setCursor(36,12);
    u8g2.print(tm.Second);
  }
  switch(tm.Wday){
    case 1:
      u8g2.drawStr(52,12,"Sun");
      break;
    case 2:
      u8g2.drawStr(52,12,"Mon");
      break;
    case 3:
      u8g2.drawStr(52,12,"Tue");
      break; 
    case 4:
      u8g2.drawStr(52,12,"Wen");
      break;  
    case 5:
      u8g2.drawStr(52,12,"Thu");
      break;  
    case 6:
      u8g2.drawStr(52,12,"Fri");
      break; 
    case 7:
      u8g2.drawStr(52,12,"Sat");
      break;
  }
  if(tm.Day < 10){
  u8g2.setCursor(73,12);
  u8g2.print(0);
  u8g2.setCursor(79,12);
  u8g2.print(tm.Day);  
  }
  else{
    u8g2.setCursor(73,12);
    u8g2.print(tm.Day);
  }
  u8g2.setFont(u8g2_font_5x7_tf);
  u8g2.drawStr(84,13,".");
  u8g2.setFont(u8g2_font_6x10_tf);
  if(tm.Month < 10){
  u8g2.setCursor(89,12);
  u8g2.print(0);
  u8g2.setCursor(95,12); 
  u8g2.print(tm.Month);     
  }
  else{
    u8g2.setCursor(89,12);
    u8g2.print(tm.Month);
  }
  u8g2.setFont(u8g2_font_5x7_tf);  
  u8g2.drawStr(100,13,".");  
  u8g2.setFont(u8g2_font_6x10_tf);  
  u8g2.setCursor(105,12);
  u8g2.print(tm.Year + 1970);
 
  if(!timeStartInfo){
    //u8g2.drawStr(30,21,"Overview | Start");
    u8g2.setFont(u8g2_font_5x8_tf);
    u8g2.drawStr(35,24,"Overview  |  Start");
    PrintBattery();    
  }
  u8g2.sendBuffer();
  timeStartInfo = false;
}



void TimeTodayPage(){
    u8g2.clearBuffer();
    if (!SD.begin(chipSelect)){
      u8g2.drawStr(0,0,"SD initialization");
      u8g2.drawStr(0,15,"failed!");
      u8g2.sendBuffer();
      storeUpdate = true;
      delay(1500);
      modus = 0;
      return;
    }
    if(!pageUpdate){
      myFile = SD.open("date.txt");
      if (myFile){
        if(calcDiff(tm.Day, tm.Month, tm.Year + 1970)){
          u8g2.clearBuffer();  
          u8g2.drawStr(0,0,"Time Today");
          u8g2.drawStr(0,10,"Hours and Minutes");
          u8g2.setCursor(0,20);
          u8g2.print(diffTime.Hour);
          u8g2.setCursor(60,20);
          u8g2.print(diffTime.Minute); 
        }
        else{
          u8g2.drawStr(0,10,"Date uncomplete.");
          u8g2.drawStr(0,20,"Edit!");
        }
        myFile.close();
      }
      else {
        u8g2.drawStr(0,26,"error opening date.txt");
      }
     u8g2.sendBuffer();
    }
    pageUpdate = true; 
}



void TimeYesterdayPage(){
    u8g2.clearBuffer();
    if (!SD.begin(chipSelect)){
      u8g2.drawStr(0,0,"SD initialization");
      u8g2.drawStr(0,15,"failed!");
      u8g2.sendBuffer();
      storeUpdate = true;
      delay(1500);
      modus = 0;
      return;
    }
    if(!pageUpdate){    
      myFile = SD.open("date.txt");
      if (myFile){
        if(calcDiff(tm.Day - 1, tm.Month, tm.Year + 1970)){
          u8g2.clearBuffer();  
          u8g2.drawStr(0,0,"Time Yesterday");
          u8g2.drawStr(0,10,"Hours and Minutes");
          u8g2.setCursor(0,20);
          u8g2.print(diffTime.Hour);
          u8g2.setCursor(60,20);
          u8g2.print(diffTime.Minute); 
        }
        else{
          u8g2.drawStr(0,10,"Date uncomplete.");
          u8g2.drawStr(0,20,"Edit!");
        }
        myFile.close();
      }
      else {
        u8g2.drawStr(0,26,"error opening date.txt");
      }
       u8g2.sendBuffer();   
    }
    pageUpdate = true; 
}


void TimeWeekPage(){
    bool fileOk = false;
    diffTime.WeekHour = 0;
    diffTime.WeekMinute = 0;
    u8g2.clearBuffer();
    if (!SD.begin(chipSelect)){
      u8g2.drawStr(0,0,"SD initialization");
      u8g2.drawStr(0,15,"failed!");
      u8g2.sendBuffer();
      storeUpdate = true;
      delay(1500);
      modus = 0;
      return;
    }
    if(!pageUpdate){   
      myFile = SD.open("date.txt");
      if (myFile){
        for(int x = 1; x < 8; x++){
          if(x == tm.Wday){
            if(x == 1)x = 8;
            for(int i = 0; i < x - 1; i++){
              if(calcDiff(tm.Day - i, tm.Month, tm.Year + 1970)){
                diffTime.WeekHour += diffTime.Hour;
                diffTime.WeekMinute += diffTime.Minute;
                fileOk = true;
                myFile.seek(0);
              }
              else{
               fileOk = false;
               break;
               }
            }
          }
        }
        if(fileOk){
          if(diffTime.WeekMinute > 60){
            diffTime.WeekHour += diffTime.WeekMinute/60;
            diffTime.WeekMinute = diffTime.WeekMinute % 60;
          }
         u8g2.clearBuffer();  
         u8g2.drawStr(0,0,"Time this Week");
         u8g2.drawStr(0,10,"Hours and Minutes");
         u8g2.setCursor(0,20);
         u8g2.print(diffTime.WeekHour);
         u8g2.setCursor(60,20);
         u8g2.print(diffTime.WeekMinute); 
        }
        else{
          u8g2.drawStr(0,10,"Date uncomplete.");
          u8g2.drawStr(0,20,"Edit!");
        }
        myFile.close();
      }
      else {
        u8g2.drawStr(0,26,"error opening date.txt");
      }
       u8g2.sendBuffer();  
    }
    pageUpdate = true;  
}



void TimeMonthPage(){
    bool fileOk = false;
    diffTime.MonthHour = 0;
    diffTime.MonthMinute = 0;    
    u8g2.clearBuffer();
    if (!SD.begin(chipSelect)){
      u8g2.drawStr(0,0,"SD initialization");
      u8g2.drawStr(0,15,"failed!");
      u8g2.sendBuffer();
      storeUpdate = true;
      delay(1500);
      modus = 0;
      return;
    }
    if(!pageUpdate){   
      myFile = SD.open("date.txt");
      if(myFile){
         for(int i = 0; i < tm.Day; i++){
           if(calcDiff(tm.Day - i, tm.Month, tm.Year + 1970)){
             diffTime.MonthHourDiagramm[i] = diffTime.Hour;
             diffTime.MonthMinuteDiagramm[i] = diffTime.Minute;
             diffTime.MonthHour += diffTime.Hour;
             diffTime.MonthMinute += diffTime.Minute;
             fileOk = true;
             myFile.seek(0);
           }
           else{
            fileOk = false;
            break;
           }
         }
       if(fileOk){
        if(!diagrammActive){
          if(diffTime.MonthMinute > 60){
            diffTime.MonthHour += diffTime.MonthMinute/60;
            diffTime.MonthMinute = diffTime.MonthMinute % 60;
          }          
          u8g2.clearBuffer();  
          u8g2.drawStr(0,0,"Time this Month");
          u8g2.drawStr(0,10,"Hours and Minutes");
          u8g2.setCursor(0,20);
          u8g2.print(diffTime.MonthHour);
          u8g2.setCursor(60,20);
          u8g2.print(diffTime.MonthMinute);          
        }
        else{  
          u8g2.clearBuffer();  
          u8g2.drawStr(0,0,"Month Diagramm");
          u8g2.drawLine(0,10,0,32);
          for(int i = 0; i < 8; i++){
            u8g2.drawLine(0, (32 - (i * 3)), 2, (32 - (i * 3))); 
          }                                                                                                          
          for(int i = 32; i >= 0 ; i--){
            int addMinute = 0;
            int addNewMinute = 0;
          //u8g2.drawLine((32 - i)*4, 31 - (diffTime.MonthHourDiagramm[i])*2,(33 - i)*4, 31 - (diffTime.MonthHourDiagramm[i - 1])*2);
            if(diffTime.MonthMinuteDiagramm[i] <= 20 && diffTime.MonthMinuteDiagramm[i] > 5)addMinute = 1;
            else if(diffTime.MonthMinuteDiagramm[i] > 20 && diffTime.MonthMinuteDiagramm[i] <= 40)addMinute = 2;
            else if(diffTime.MonthMinuteDiagramm[i] > 40 && diffTime.MonthMinuteDiagramm[i] < 60)addMinute = 3;
            else addMinute = 0;
            if(diffTime.MonthMinuteDiagramm[i - 1] <= 20 && diffTime.MonthMinuteDiagramm[i - 1] > 5)addNewMinute = 1;
            else if(diffTime.MonthMinuteDiagramm[i - 1] > 20 && diffTime.MonthMinuteDiagramm[i - 1] <= 40)addNewMinute = 2;
            else if(diffTime.MonthMinuteDiagramm[i - 1] > 40 && diffTime.MonthMinuteDiagramm[i - 1] < 60)addNewMinute = 3;
            else addNewMinute = 0;
            if(diffTime.MonthHourDiagramm[i] >= 9)diffTime.MonthHourDiagramm[i] = 9;
            u8g2.drawLine((32 - i)*4, 31 - (diffTime.MonthHourDiagramm[i])*3 - addMinute,(32 - i)*4, 31 - (diffTime.MonthHourDiagramm[i - 1])*3 - addNewMinute);
            u8g2.drawLine((32 - i)*4, 31 - (diffTime.MonthHourDiagramm[i - 1])*3 - addNewMinute, (32 - i)*4 + 4, 31 - (diffTime.MonthHourDiagramm[i - 1])*3 - addNewMinute);
            delay(4);
            u8g2.sendBuffer();
          }  
         }
        }
        else{
           u8g2.drawStr(0,10,"Date uncomplete.");
           u8g2.drawStr(0,20,"Edit!");
         }
         myFile.close();
       }
       else{
         u8g2.drawStr(0,26,"error opening date.txt");
       }
        u8g2.sendBuffer();  
    }
    pageUpdate = true;  
}



void StartFunction(){
  getTime();
  if(tm.Hour == 23 && tm.Minute == 59 && tm.Second == 59)storeUpdate = false;
  if(tm.Hour == 0 && tm.Minute == 0 && tm.Second == 1)storeUpdate = false;
  u8g2.clearBuffer();
  if (!SD.begin(chipSelect)){
    u8g2.drawStr(0,0,"SD initialization");
    u8g2.drawStr(0,15,"failed!");
    u8g2.sendBuffer();
    storeUpdate = true;
    delay(1500);
    modus = 0;
    timeStart = false;
    return;
  }  
  if(!storeUpdate){
    SDWriteData();
    storeUpdate = true;
  }
  if(blink)analogWrite(ledPin,50);
  else analogWrite(ledPin,0);
  blink = !blink;
  secondCounter++;

  if(secondCounter == 60){
    workingMinute++;
    secondCounter = 0;
  }
  if(workingMinute == 60){
    workingHour++;    
    workingMinute = 0;
  }

  u8g2.clearBuffer();
  u8g2.drawStr(0,0,"you are working since");
  if(workingHour < 10){
    u8g2.setCursor(40,13);
    u8g2.print(0);
    u8g2.setCursor(46,13);
    u8g2.print(workingHour);
  }
  else{
    u8g2.setCursor(40,13);
    u8g2.print(workingHour);
  }
  u8g2.setFont(u8g2_font_5x8_tf);
  u8g2.drawStr(52,13,":");
  u8g2.setFont(u8g2_font_6x10_tf);
  if(workingMinute < 10){
    u8g2.setCursor(58,13);
    u8g2.print(0);
    u8g2.setCursor(64,13);
    u8g2.print(workingMinute);
  }
  else{
    u8g2.setCursor(58,13);
    u8g2.print(workingMinute);
  }
  u8g2.setFont(u8g2_font_5x8_tf);  
  u8g2.drawStr(70,13,":");
  u8g2.setFont(u8g2_font_6x10_tf);  
  if(secondCounter < 10){
    u8g2.setCursor(76,13);
    u8g2.print(0);
    u8g2.setCursor(82,13);
    u8g2.print(secondCounter);
  }
  else{
    u8g2.setCursor(76,13);
    u8g2.print(secondCounter);
  }
  u8g2.setFont(u8g2_font_5x8_tf);
  u8g2.drawStr(47,24,"Info  |  Stop");
  PrintBattery();
  u8g2.sendBuffer();
}



void StopFunction(){
    getTime();
    secondCounter = 0;
    workingMinute = 0;
    workingHour = 0;
    u8g2.clearBuffer();
    if (!SD.begin(chipSelect)){
      u8g2.drawStr(0,0,"SD initialization");
      u8g2.drawStr(0,15,"failed!");
      u8g2.sendBuffer();
      delay(1500);
      modus = 0;
      return;
    }  
    SDWriteData();
    storeUpdate = true;
    analogWrite(ledPin,0);
    secondCounter = 0;
    u8g2.clearBuffer();  
    u8g2.drawStr(10,15,"have a nice time!");
    u8g2.sendBuffer();
    delay(800);
}



bool calcDiff(int Day, int Month, int Year){
    bool firstHit = false;
    int counter = 0;
    diffTime.Hour = 0;
    diffTime.Minute = 0;
    int fIDay = Day/10;
    int sIDay = (Day % 10);
    int fIMonth = Month/10;
    int sIMonth = Month % 10;
    int fIYear = (Year % 100)/10;
    int sIYear = Year % 10;      
    while(readStream(myFile)){            
        if((lineBuffer[0]-'0') == fIDay && (lineBuffer[1]-'0') == sIDay && (lineBuffer[3]-'0') == fIMonth
            && (lineBuffer[4]-'0') == sIMonth && (lineBuffer[8]-'0') == fIYear && (lineBuffer[9]-'0') == sIYear){
          counter++;
          if(firstHit == true && (counter % 2) == 0){
            if(((lineBuffer[fIHour] - '0') == (oldLineBuffer[fIHour] - '0')) && ((lineBuffer[sIHour] - '0') == (oldLineBuffer[sIHour] - '0'))){
              diffTime.Minute += ((lineBuffer[fIMinute] - '0') - (oldLineBuffer[fIMinute] - '0'))*10 + ((lineBuffer[sIMinute] - '0') - (oldLineBuffer[sIMinute] - '0'));
            }
            else{
              if(((lineBuffer[fIHour] - '0') - (oldLineBuffer[fIHour] - '0'))*10 + ((lineBuffer[sIHour] - '0') - (oldLineBuffer[sIHour] - '0')) == 1){
                diffTime.Minute += (((lineBuffer[fIMinute] - '0')*10 + (lineBuffer[sIMinute] - '0')) + (60 - ((oldLineBuffer[fIMinute] - '0')*10 + (oldLineBuffer[sIMinute] - '0'))));
              } //diffTime.Hour - 1;
              else{
                if((lineBuffer[fIMinute] - '0') == (oldLineBuffer[fIMinute] - '0') && (lineBuffer[sIMinute] - '0') == (oldLineBuffer[sIMinute] - '0')){
                diffTime.Hour += ((lineBuffer[fIHour] - '0') - (oldLineBuffer[fIHour] - '0'))*10 + ((lineBuffer[sIHour] - '0') - (oldLineBuffer[sIHour] - '0'));
                }
                else{
                diffTime.Hour += ((lineBuffer[fIHour] - '0') - (oldLineBuffer[fIHour] - '0'))*10 + ((lineBuffer[sIHour] - '0') - (oldLineBuffer[sIHour] - '0')) - 1;
                diffTime.Minute += (((lineBuffer[fIMinute] - '0')*10 + (lineBuffer[sIMinute] - '0')) + (60 - ((oldLineBuffer[fIMinute] - '0')*10 + (oldLineBuffer[sIMinute] - '0'))));
                }
              }
            }    
          }
        }
        for(int i = 0; i < sizeof(lineBuffer); i++){
          oldLineBuffer[i] = lineBuffer[i];
        }
        firstHit = true;
    }
    if((counter % 2) == 0){
      if(diffTime.Minute > 60){
        diffTime.Hour += diffTime.Minute/60;
        diffTime.Minute = diffTime.Minute % 60;
      }
      return true;
    }
    else{
       diffTime.Hour = 0;
       diffTime.Minute = 0;
       return false;  
    }
}



bool readStream(Stream& stream){
  static byte i;
  while (stream.available()){
    char c = stream.read();
    if(c >= 32 && i < (lineSize - 1)){
      lineBuffer[i++] = c;
    }
    else if (c == '\n' && i > 0){
      lineBuffer[i] = '\0';
      i = 0;
      return true;
    }
  }
  return false;
}



void SDWriteData(){
    myFile = SD.open("date.txt", FILE_WRITE);    
    if(myFile){ 
      if(tm.Day < 10){
        myFile.print(0);
        myFile.print(tm.Day);
      }else myFile.print(tm.Day);
      myFile.print(".");
      if(tm.Month < 10){
        myFile.print(0);
        myFile.print(tm.Month);
      }else myFile.print(tm.Month);
      myFile.print(".");
      myFile.print(tm.Year + 1970);
      myFile.print("-");
      if(tm.Hour < 10){
        myFile.print(0);
        myFile.print(tm.Hour);
      }else myFile.print(tm.Hour);
      myFile.print(":");
      if(tm.Minute < 10){
        myFile.print(0);
        myFile.println(tm.Minute);
      }else myFile.println(tm.Minute);
      myFile.close(); 
    }      
}


void CheckTimeToday(){
 u8g2.clearBuffer();
    if (!SD.begin(chipSelect)){
      u8g2.drawStr(0,0,"SD initialization");
      u8g2.drawStr(0,15,"failed!");
      u8g2.sendBuffer();
      storeUpdate = true;
      delay(1000);
      return;
    }
      myFile = SD.open("date.txt");
      if (myFile){
        if(calcDiff(tm.Day, tm.Month, tm.Year + 1970)){
        }
        else{
          u8g2.drawStr(0,10,"Date uncomplete.");
          u8g2.drawStr(0,20,"Edit!");
        }
        myFile.close();
      }
      else {
        u8g2.drawStr(0,26,"error opening date.txt");
      }
     u8g2.sendBuffer();
     checkTimeBeforeStart = true;     
}


void PrintTimeTodayPage(){
   u8g2.clearBuffer();  
   u8g2.drawStr(0,0,"Time Today");
   u8g2.drawStr(0,10,"Hours and Minutes");
   u8g2.setCursor(0,20);
   u8g2.print(diffTime.Hour);
   u8g2.setCursor(60,20);
   u8g2.print(diffTime.Minute);
   u8g2.sendBuffer();
}


void PrintBattery(){
  int currentBattery;
  currentBattery = battery / 2;
  u8g2.setFont(u8g2_font_micro_tr);
  u8g2.setCursor(3,24);
  u8g2.print(currentBattery);
  if(currentBattery < 100)u8g2.drawStr(13,24,"%");
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.drawLine(0,31,0,23);
  u8g2.drawLine(0,23,18,23);
  u8g2.drawLine(18,23,18,28);
  u8g2.drawLine(18,26,20,26);
  u8g2.drawLine(20,26,20,28);
  u8g2.drawLine(20,28,18,28);
  u8g2.drawLine(18,28,18,31);
  u8g2.drawLine(18,31,0,31);
}



void initLogo(int offsetX, int offsetY, int animationSpeed){
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_unifont_t_greek); //u8g2_font_9x15_tf  8x13 7x14tf u8g2_font_unifont_t_greek
  u8g2.setCursor(0,20);
  u8g2.print(title);
  u8g2.drawCircle(offsetX,offsetY,10, U8G2_DRAW_ALL);
  u8g2.drawCircle(offsetX - 6, offsetY - 11, 2);
  u8g2.drawCircle(offsetX + 6, offsetY - 11 , 2);
  u8g2.drawLine(offsetX - 6, offsetY + 10, offsetX - 8, offsetY + 13);
  u8g2.drawLine(offsetX + 6, offsetY + 10, offsetX + 8, offsetY + 13);
  u8g2.drawLine(offsetX,offsetY,offsetX, offsetY - 8);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 1, offsetY - 8);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 2, offsetY - 7);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 3, offsetY - 7);
  u8g2.sendBuffer();
  delay(animationSpeed);  
  u8g2.drawLine(offsetX,offsetY,offsetX + 4, offsetY - 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 5, offsetY - 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 5, offsetY - 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 6, offsetY - 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 6, offsetY - 4);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 7, offsetY - 3);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 7, offsetY - 2);
  u8g2.sendBuffer(); 
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 8, offsetY - 1);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 8, offsetY);
  u8g2.sendBuffer();
  delay(animationSpeed);   

  u8g2.drawLine(offsetX,offsetY,offsetX + 8, offsetY + 1);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 7, offsetY + 2);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 7, offsetY + 3);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 6, offsetY + 4);
  u8g2.sendBuffer();
  delay(animationSpeed);  
  u8g2.drawLine(offsetX,offsetY,offsetX + 6, offsetY + 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 5, offsetY + 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 5, offsetY + 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 4, offsetY + 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 3, offsetY + 7);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 2, offsetY + 7);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX + 1, offsetY + 8);
  u8g2.sendBuffer(); 
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX, offsetY + 8);
  u8g2.sendBuffer();
  delay(animationSpeed);


  u8g2.drawLine(offsetX,offsetY,offsetX - 1, offsetY + 8);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 2, offsetY + 7);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 3, offsetY + 7);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 4, offsetY + 6);
  u8g2.sendBuffer();
  delay(animationSpeed);  
  u8g2.drawLine(offsetX,offsetY,offsetX - 5, offsetY + 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 5, offsetY + 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 6, offsetY + 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 6, offsetY + 4);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 7, offsetY + 3);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 7, offsetY + 2);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 8, offsetY + 1);
  u8g2.sendBuffer(); 
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 8, offsetY);
  u8g2.sendBuffer();
  delay(animationSpeed);    

  u8g2.drawLine(offsetX,offsetY,offsetX - 8, offsetY - 1);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 7, offsetY - 2);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 7, offsetY - 3);
  u8g2.sendBuffer();  
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 6, offsetY - 4);
  u8g2.sendBuffer();
  delay(animationSpeed);  
  u8g2.drawLine(offsetX,offsetY,offsetX - 6, offsetY - 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 5, offsetY - 5);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 5, offsetY - 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 4, offsetY - 6);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 3, offsetY - 7);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 2, offsetY - 7);
  u8g2.sendBuffer();
  delay(animationSpeed);
  u8g2.drawLine(offsetX,offsetY,offsetX - 1, offsetY - 8);
  u8g2.sendBuffer();
  delay(100);
}
